/* ************************************************************************* *
   jlmem - (C) Copyright 1996 Robert Gasch (Robert_Gasch@peoplesoft.com)

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */

import java.applet.*;
import java.lang.*;
import java.io.*;
import java.awt.*;
import java.net.*;


/* ************************************************************************** */
/* *** jlmem: Java Linux MEmor Monitor: main class definition *************** */
/* ************************************************************************** */
public class jlmem extends Frame implements Runnable 
{
	private DrawLinuxMem 		dlm;
	private long			turns=0;
	private boolean			done=false,
					absolute=false,
					verbose=false;
	private int			interval=60000,	/* 1 minute */
					mode, old_mode;
	private Frame 			f;
	private Panel 			p=null;
	private Graphics		g=null;
	private MenuBar			mbar;
	private Menu			m_file, m_window, m_about;
	private MenuItem		mi_main, mi_mem, mi_swap, mi_about;
	private Thread			t=null;
	private Button			OK=null;
	private static final int 	WIDTH=250, 
					OFFY=50,
					HEIGHT=180,
					MARGIN=15,
					MODE_MAIN_WIN=0,
					MODE_DETAIL_MEM=1,
					MODE_DETAIL_SWAP=2,
					MODE_ABOUT=3;
	private static final String 	version=new String ("jlmem v1.03"),
					copyright=new String ("Copyright by Robert Gasch"),
					email=new String ("Robert_Gasch@peoplesoft.com");

	/** init: initialize jlmem, create user interface ... */
	public void init ()
		{
		if (verbose)
			{
			System.out.print (version);
			System.out.print (" - ");
			System.out.print (copyright);
			System.out.println (" (" + email + ")");
			}
		/** Do the basic GUI initialization */
		f=new Frame (version);
		f.setFont(new Font("TimesRoman", Font.PLAIN, 14));
		f.setForeground(Color.blue);
		f.setLayout(new BorderLayout());
		mode=MODE_MAIN_WIN;

		/** Compose the menus */
		mbar=new MenuBar ();
		m_file=new Menu ("File");
		m_file.add(new MenuItem ("Update"));
		m_file.addSeparator ();
		m_file.add(new MenuItem ("Exit"));
		mbar.add (m_file);

		m_window=new Menu ("Screen");
		mi_main = new MenuItem ("Main Window");
		mi_main.disable();
		m_window.add(mi_main);
		mi_mem = new MenuItem ("Detail - Mem");
		m_window.add(mi_mem);
		mi_swap = new MenuItem ("Detail - Swap");
		m_window.add(mi_swap);
		mbar.add (m_window);

		m_about=new Menu ("About...");
		mi_about = new MenuItem ("About jlmem");
		m_about.add (mi_about);
		mbar.add (m_about);
		//mbar.setHelpMenu (m_about);

		setMenuBar (mbar);

		/** Create the update thread */
		if (t == null)
			{
			t=new Thread (this, "Update");
			t.start();
			}

		/** Adjust size and show the GUI */
		resize (WIDTH, HEIGHT);
		show();
		}

	
	/** main: do the main stuff */
	public static void main (String[] args)
		{
		jlmem j=new jlmem();

		j.ParseArgs (args);
		j.init ();
		}


	/** run: main loop for the update thread; */
	/** called automatically after thread creation */
	public void run ()
		{
		while (Thread.currentThread() == t)
			{
			dlm=null;
			dlm=new DrawLinuxMem(verbose, absolute,
					     WIDTH, HEIGHT, OFFY, MARGIN);
			repaint ();
			verbose=false;
			turns++;
			try 
				{
				t.sleep (interval);
				}
			catch (InterruptedException e)
				{
				}
			}
		}


	/** stop: kill the tread once our applet is dead */
	/** called automatically when the thread exits */
	public void stop ()
		{
		t=null;
		}


	/** ParseArgs: parse program aruments */
	public void ParseArgs (String[] args)
		{
		for (int i=0; i<args.length; i++)
			{
			if (args[i].equals ("-?") || args[i].equals ("-help"))
				{
				System.out.println ("jlmem  - Java Linux MEmory Monitor");
				usage();
				}
			else
			if (args[i].equals ("-v"))
				{
				verbose=true;
				System.out.println ("verbose = " + verbose);
				}
			else
			if (args[i].equals ("-a"))
				{
				absolute=true;
				System.out.println ("absolute = " + verbose);
				}
			else
			if (args[i].equals ("-i"))
				{
				if (args.length == i+1)
					{
					System.out.println ("Missing value parameter after -i ... Exiting");
					usage ();
					}
				Integer t=new Integer (args[++i]);
				System.out.println ("interval = " + t + " seconds");
				interval = (t.intValue()*1000);
				}
			else
				{
				System.out.println ("Unrecognized arg [" + args[i] + "] ... Exiting");
				usage ();
				}
			}
		}


	/** usage: show the proper usage of jlmem */
	public static void usage ()
		{
		System.out.println ("Usage:   jlmem [<option> [<value>]]");
		System.out.println ("Options: -i <value>: update statistics every <value> seconde");
		System.out.println ("       : -a: draw bars in absolute size");
		System.out.println ("       : -v: show verbose info on first turn");
		System.out.println ("       : -?: show this info");
		System.exit (0);
		}


	/** paint: draw the actual memory graphs */
	/** called through repaint() or automatically */
	public void paint (Graphics g)
		{
		Double	x;
		String 	s1, s2;
		int	i, off=2;

		this.g=g;
		//System.out.println ("Painting ...");
		/** for some off reason this is not taken care of automatically */
		if (old_mode==MODE_ABOUT)
			{
			g.clearRect (0, 0, WIDTH, HEIGHT);
			resize (WIDTH, HEIGHT);
			show();
			}
		if (mode == MODE_MAIN_WIN)
			{
			dlm.DrawGraph (g);
			}
		else
		if (mode == MODE_DETAIL_MEM)
			{
			dlm.DrawMain (g);
			}
		else
		if (mode == MODE_DETAIL_SWAP)
			{
			dlm.DrawSwap (g);
			}
		else
		if (mode == MODE_ABOUT)
			{
			g.clearRect (0, 0, WIDTH, HEIGHT);
			show();
			g.drawString (version, MARGIN, OFFY+MARGIN*2); 
			g.drawString (copyright, MARGIN, OFFY+MARGIN*3); 
			g.drawString (email, MARGIN, OFFY+MARGIN*4); 
			}
		}


	/** action: handle a user-generated event */
	/** Called automatically when user event is generated */
	public boolean action (Event e, Object o)
		{
		//System.out.println (e);
		// Handle the Menus
		if (e.target instanceof MenuItem)
			{
			if ("Update".equals (o.toString()))
				{
				dlm=null;
				dlm=new DrawLinuxMem(verbose, absolute,
						     WIDTH, HEIGHT, OFFY, MARGIN);
				repaint ();
				verbose=false;
				turns++;
				}
			else
			if ("Main Window".equals (o.toString()))
				{
				mi_main.disable ();
				mi_mem.enable ();
				mi_swap.enable ();
				mode = MODE_MAIN_WIN;
				}
			else
			if ("Detail - Mem".equals (o.toString()))
				{
				mi_main.enable ();
				mi_mem.disable ();
				mi_swap.enable ();
				mode = MODE_DETAIL_MEM;
				}
			else
			if ("Detail - Swap".equals (o.toString()))
				{
				mi_main.enable ();
				mi_mem.enable ();
				mi_swap.disable ();
				mode = MODE_DETAIL_SWAP;
				}
			else
			if ("About jlmem".equals (o.toString()))
				{
				mi_about.disable ();
				mi_main.disable();
				mi_mem.disable();
				mi_swap.disable();
				if (p==null)
					{
					p=new Panel();
					OK=new Button ("Ok");
					}
				p.add (OK);
				this.add("South", p);
				old_mode=mode;
				mode=MODE_ABOUT;
				show();
				}
			else
			if ("Exit".equals (o.toString()))
				{
				//System.out.println ("Exit");
				System.exit (0);
				}
			else
				{
				System.out.println ("Unhandled Menu " + o.toString() + " ... Exiting");
				System.exit (1);
				}
			}
		// Handle the Buttons 
		if (e.target instanceof Button)
			{
			if ("Ok".equals (o.toString()))
				{
				mi_about.enable ();
				if (old_mode != MODE_MAIN_WIN)
					mi_main.enable();
				if (old_mode != MODE_DETAIL_MEM)
					mi_mem.enable();
				if (old_mode != MODE_DETAIL_SWAP)
					mi_swap.enable();
				this.remove (p);
				mode=old_mode;
				old_mode=MODE_ABOUT;
				show();
				}
			}
		repaint ();
		return (true);
		}
}



/* ************************************************************************** */
/* *** DrawLinuxMem: Superclass to ReadLinuxMem to draw different screens *** */
/* ************************************************************************** */
class DrawLinuxMem extends ReadLinuxMem
{
	private boolean			verbose,
					absolute;
	private int			WIDTH, OFFY, HEIGHT, MARGIN;

	/** DrawLinuxMem: constructor: do everything necessary */
	public DrawLinuxMem (	boolean _verbose_, 
				boolean _absolute_, 
				int	_WIDTH_,
				int	_HEIGHT_, 
				int	_OFFY_,
				int	_MARGIN_)
		{
		super (_verbose_);
		this.absolute=_absolute_;
		this.verbose=_verbose_;
		this.WIDTH=_WIDTH_;
		this.HEIGHT=_HEIGHT_;
		this.OFFY=_OFFY_;
		this.MARGIN=_MARGIN_;
		}


	/** DrawLong: draw a long into the specified Graphics at given y-offset */
	public void DrawLong (Graphics g, long l, String label, int off)
		{
		Long	L=new Long (l);
		String	S=L.toString();

		g.drawString (label, MARGIN, OFFY+MARGIN*off);
		g.drawString (S, MARGIN+65+(100-S.length()*8), OFFY+MARGIN*off);
		}


	/** DrawGraph: draw the linux memory info in bar graph form */
	public void DrawGraph (Graphics g)
		{
		Double		x, real_width, swap_width;
		double		d1, d2;
		String 		s1, s2, m1, m2;                        
                Long            y;

		g.drawString ("Real", MARGIN, OFFY+MARGIN*2-2);
	        
		y = new Long(this.main_mem/1000);                          /* Draw the amount of main memory on main Screen */
	        m1 = y.toString();                                         /* Implemented by Anderson Silva            */
		g.drawString (m1+" KB", MARGIN+35, OFFY+MARGIN*2-2);
		
		if (absolute)
			real_width=new Double (WIDTH-MARGIN*2);
		else
			{
			d1=main_mem;
			d2=biggest_mem;
			real_width=new Double ((d1/d2)*(WIDTH-MARGIN*2));
			}
		g.drawRect (MARGIN, OFFY+MARGIN*2, real_width.intValue(), MARGIN);
		x=new Double (real_width.intValue()*this.mem_perc);
		g.fillRect (MARGIN, OFFY+MARGIN*2, x.intValue(), MARGIN);
		x=new Double (this.mem_perc*100);
		s1=x.toString ();
		s2=s1.substring (0, s1.length()%5);
		g.drawString (s2+"%", (WIDTH-MARGIN)-s2.length()*15, OFFY+MARGIN*2-2); 

		
		y = new Long(this.swap_mem/1000);                         /* Draw the amount of swap memory on main Screen */
	        m2 = y.toString();                                        /* Implemented by Anderson Silva            */
		g.drawString (m2+" KB", MARGIN+35, OFFY+MARGIN*5-2);

		
		g.drawString ("Swap", MARGIN, OFFY+MARGIN*5-2);
		if (absolute)
			swap_width=new Double (WIDTH-MARGIN*2);
		else
			{
			d1=swap_mem;
			d2=biggest_mem;
			swap_width=new Double ((d1/d2)*(WIDTH-MARGIN*2));
			}
		g.drawRect (MARGIN, OFFY+MARGIN*5, swap_width.intValue(), MARGIN);
		x=new Double (swap_width.intValue()*this.swap_perc);
		if (this.swap_used > 0)
			g.fillRect (MARGIN, OFFY+MARGIN*5, x.intValue(), MARGIN);
		x=new Double (this.swap_perc*100);
		s1=x.toString ();
		s2=s1.substring (0, s1.length()%5);
		g.drawString (s2+"%", (WIDTH-MARGIN)-s2.length()*15, OFFY+MARGIN*5-2); 
		}

	
	/** DrawMain: Write out the info for the main memory */
	public void DrawMain (Graphics g)
		{
		int	off=2;

		this.DrawLong (g, this.main_mem, "Memory", off++);
		this.DrawLong (g, this.free_mem, "Mem Free", off++);
		this.DrawLong (g, this.used_mem, "Mem Used", off++);
		this.DrawLong (g, this.shared_mem, "Shared", off++);
		this.DrawLong (g, this.buf_mem, "Buffered", off++);
		this.DrawLong (g, this.cached_mem, "Cached", off++);
		}


	/** DrawSwap: Write out the info for swap space */
	public void DrawSwap (Graphics g)
		{
		int	off=2;

		this.DrawLong (g, this.swap_mem, "Swap", off++);
		this.DrawLong (g, this.swap_free, "Swap Free", off++);
		this.DrawLong (g, this.swap_used, "Swap Used", off++);
		}
}



/* *********************************************************************** */
/* *** ReadLinuxMem: Class to get and hold the info from /proc/meminfo *** */
/* *********************************************************************** */
class ReadLinuxMem 
{
	public long		main_mem, 
				used_mem, 
				free_mem, 
				shared_mem, 
				buf_mem,
				cached_mem,
				swap_mem, 
				swap_used, 
				swap_free,
				biggest_mem; 
	public double		mem_perc,
				swap_perc;
	private boolean		verbose;
	private DataInputStream	d;


	/** ReadLinuxMem: constructor: do everything necessary */
	public ReadLinuxMem (boolean _verbose_)
		{
		this.verbose=_verbose_;
		d=this.OpenFile ("/proc/meminfo");
		this.ParseLinuxMem ();
		this.CloseFile (d);
		}


	/** ParseLinuxMem: read and parse /proc/meminfo */
	public void ParseLinuxMem ()
		{
		String	line;

		try 
			{
			StreamTokenizer		t=new StreamTokenizer (d);
			Double 			dbl;

			// Skip first line
			line = d.readLine ();
			if (verbose)
				System.out.println (line);

			// Read "Mem:" line
			t.nextToken ();
			// Sanity Check
			if (t.sval.compareTo ("Mem") != 0)
				{
				System.out.println 
					("Sanity Check failed in 2nd line: Expected [Mem], found [" 
						+ t.sval + "] ... Exiting");
				System.exit (2);
				}
			// Parse and assign memory values
			t.nextToken ();
			main_mem=TokenToLong(t);
			used_mem=TokenToLong(t);
			free_mem=TokenToLong(t);
			shared_mem=TokenToLong(t);
			buf_mem=TokenToLong(t);
			cached_mem=TokenToLong(t);
			if (verbose)
				{
				System.out.println ("Main="+main_mem);
				System.out.println ("Used="+used_mem);
				System.out.println ("Free="+free_mem);
				System.out.println ("Shared="+shared_mem);
				System.out.println ("Buf="+buf_mem);
				System.out.println ("Cached="+cached_mem);
				}

			// Parse and assign Swap values
			t.nextToken ();
			// Sanity Check
			if (t.sval.compareTo ("Swap") != 0)
				{
				System.out.println 
					("Sanity Check failed in 3rd line: Expected [Swap], found [" 
						+ t.sval + "] ... Exiting");
				System.exit (2);
				}
			t.nextToken ();
			biggest_mem=swap_mem=TokenToLong(t);
			swap_used=TokenToLong(t);
			swap_free=TokenToLong(t);
			if (verbose)
				{
				System.out.println ("SwapMem="+swap_mem);
				System.out.println ("SwapUsed="+swap_used);
				System.out.println ("SwapFree="+swap_free);
				}

			if (main_mem > swap_mem)
				biggest_mem=main_mem;

			mem_perc=1-((double)(free_mem+buf_mem)/(double)main_mem);
			//System.out.println ("mp="+mem_perc);
			if (swap_mem > 0)
				swap_perc=1-(double)swap_free/(double)swap_mem;
			else
				swap_perc=0;
			}
		catch (IOException e)
			{
			}
		}

	
	/** TokenToLong: Read a Token and convert it to a long */
	public long TokenToLong (StreamTokenizer t) throws IOException
		{
		Double 		dbl;

		t.nextToken ();
		dbl=new Double (t.nval);
		return (dbl.longValue());
		}


	/** Open_File: open the file, assign d=DIS or handle exception */
	public DataInputStream OpenFile (String fname)
		{
		DataInputStream		d=null;

		if (verbose)
			System.out.print ("Attempting to open " + fname + " ... ");
		try 	
			{
			d = new DataInputStream (new FileInputStream(fname));
			}
		catch	(IOException e)
			{
			System.err.println ("Error opening /proc/meminfo. Is this a LINUX system?? ... Exiting");
			System.err.println (e);
			System.exit (1);
			}
		finally 
			{
			if (verbose)
				System.out.println ("Success");
			return (d);
			}
		}


	/** CloseFile: close the file pointed to by d */
	public void CloseFile (DataInputStream d)
		{
		try
			{
			d.close ();
			}
		catch (IOException e)
			{
			System.err.println (e);
			System.exit (1);
			}
		}
}







